@@ -23,7 +23,7 @@
  */
 #ifndef lint
 static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/addrtoname.c,v 1.108.2.5 2005/04/25 08:43:05 guy Exp $ (LBL)";
+    "@(#) $Header: /tcpdump/master/tcpdump/addrtoname.c,v 1.108.2.8 2006/02/27 07:27:16 hannes Exp $ (LBL)";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -68,6 +68,10 @@ extern int ether_ntohost(char *, const struct ether_addr *);
 #include "extract.h"
 #include "oui.h"
 
+#ifndef ETHER_ADDR_LEN
+#include "ether.h"
+#endif
+
 /*
  * hash tables for whatever-to-name translations
  *
@@ -88,7 +92,6 @@ struct hnamemem tporttable[HASHNAMESIZE];
 struct hnamemem uporttable[HASHNAMESIZE];
 struct hnamemem eprototable[HASHNAMESIZE];
 struct hnamemem dnaddrtable[HASHNAMESIZE];
-struct hnamemem llcsaptable[HASHNAMESIZE];
 struct hnamemem ipxsaptable[HASHNAMESIZE];
 
 #if defined(INET6) && defined(WIN32)
@@ -461,9 +464,10 @@ lookup_protoid(const u_char *pi)
 const char *
 etheraddr_string(register const u_char *ep)
 {
-	register u_int i, oui;
+	register int i;
 	register char *cp;
 	register struct enamemem *tp;
+	int oui;
 	char buf[BUFSIZE];
 
 	tp = lookup_emem(ep);
@@ -475,9 +479,9 @@ etheraddr_string(register const u_char *ep)
 
 		/*
 		 * We don't cast it to "const struct ether_addr *"
-		 * because some systems don't modify the Ethernet
-		 * address but fail to declare the second argument
-		 * as a "const" pointer.
+		 * because some systems fail to declare the second
+		 * argument as a "const" pointer, even though they
+		 * don't modify what it points to.
 		 */
 		if (ether_ntohost(buf2, (struct ether_addr *)ep) == 0) {
 			tp->e_name = strdup(buf2);
@@ -486,20 +490,20 @@ etheraddr_string(register const u_char *ep)
 	}
 #endif
 	cp = buf;
-        oui=EXTRACT_24BITS(ep);
+	oui = EXTRACT_24BITS(ep);
 	*cp++ = hex[*ep >> 4 ];
 	*cp++ = hex[*ep++ & 0xf];
-        for (i = 5; (int)--i >= 0;) {
-            *cp++ = ':';
-            *cp++ = hex[*ep >> 4 ];
-            *cp++ = hex[*ep++ & 0xf];
-        }
+	for (i = 5; --i >= 0;) {
+		*cp++ = ':';
+		*cp++ = hex[*ep >> 4 ];
+		*cp++ = hex[*ep++ & 0xf];
+	}
 
-        if (!nflag) {
-            snprintf(cp,BUFSIZE," (oui %s)",
-                     tok2str(oui_values,"Unknown",oui));
-        } else
-            *cp = '\0';
+	if (!nflag) {
+		snprintf(cp, BUFSIZE - (2 + 5*3), " (oui %s)",
+		    tok2str(oui_values, "Unknown", oui));
+	} else
+		*cp = '\0';
 	tp->e_name = strdup(buf);
 	return (tp->e_name);
 }
@@ -585,25 +589,6 @@ protoid_string(register const u_char *pi)
 	return (tp->p_name);
 }
 
-const char *
-llcsap_string(u_char sap)
-{
-	register struct hnamemem *tp;
-	register u_int32_t i = sap;
-	char buf[sizeof("sap 00")];
-
-	for (tp = &llcsaptable[i & (HASHNAMESIZE-1)]; tp->nxt; tp = tp->nxt)
-		if (tp->addr == i)
-			return (tp->name);
-
-	tp->addr = i;
-	tp->nxt = newhnamemem();
-
-	snprintf(buf, sizeof(buf), "sap %02x", sap & 0xff);
-	tp->name = strdup(buf);
-	return (tp->name);
-}
-
 #define ISONSAP_MAX_LENGTH 20
 const char *
 isonsap_string(const u_char *nsap, register u_int nsap_length)
@@ -872,40 +857,6 @@ init_etherarray(void)
 	}
 }
 
-static struct tok llcsap_db[] = {
-	{ LLCSAP_NULL,		"null" },
-	{ LLCSAP_8021B_I,	"802.1b-gsap" },
-	{ LLCSAP_8021B_G,	"802.1b-isap" },
-	{ LLCSAP_IP,		"ip-sap" },
-	{ LLCSAP_PROWAYNM,	"proway-nm" },
-	{ LLCSAP_8021D,		"802.1d" },
-	{ LLCSAP_RS511,		"eia-rs511" },
-	{ LLCSAP_ISO8208,	"x.25/llc2" },
-	{ LLCSAP_PROWAY,	"proway" },
-	{ LLCSAP_SNAP,		"snap" },
-	{ LLCSAP_IPX,		"IPX" },
-	{ LLCSAP_NETBEUI,	"netbeui" },
-	{ LLCSAP_ISONS,		"iso-clns" },
-	{ LLCSAP_GLOBAL,	"global" },
-	{ 0,			NULL }
-};
-
-static void
-init_llcsaparray(void)
-{
-	register int i;
-	register struct hnamemem *table;
-
-	for (i = 0; llcsap_db[i].s != NULL; i++) {
-		table = &llcsaptable[llcsap_db[i].v];
-		while (table->name)
-			table = table->nxt;
-		table->name = llcsap_db[i].s;
-		table->addr = llcsap_db[i].v;
-		table->nxt = newhnamemem();
-	}
-}
-
 static struct tok ipxsap_db[] = {
 	{ 0x0000, "Unknown" },
 	{ 0x0001, "User" },
@@ -1162,7 +1113,6 @@ init_addrtoname(u_int32_t localnet, u_int32_t mask)
 	init_etherarray();
 	init_servarray();
 	init_eprotoarray();
-	init_llcsaparray();
 	init_protoidarray();
 	init_ipxsaparray();
 }
