@@ -21,7 +21,7 @@
 
 #ifndef lint
 static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-frag6.c,v 1.16.2.3 2003/11/19 00:35:43 guy Exp $";
+    "@(#) $Header: /tcpdump/master/tcpdump/print-frag6.c,v 1.19.2.1 2005/04/20 22:33:21 guy Exp $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -45,14 +45,10 @@ frag6_print(register const u_char *bp, register const u_char *bp2)
 {
 	register const struct ip6_frag *dp;
 	register const struct ip6_hdr *ip6;
-	register const u_char *ep;
 
 	dp = (const struct ip6_frag *)bp;
 	ip6 = (const struct ip6_hdr *)bp2;
 
-	/* 'ep' points to the end of available data. */
-	ep = snapend;
-
 	TCHECK(dp->ip6f_offlg);
 
 	if (vflag) {
